/*
 * Match.java
 *
 * Class to abstract an Apama match event used by the stockwatch application.
 * The match event gives details of the user for whom the match is 
 * applicable, the name of the stock for which the price floor was crossed, 
 * and the traded price which fell below the floor.
 *
 * $Copyright(c) 2002,  2004 - 2006 Progress Software Corporation. All Rights Reserved.$ 
 * $Copyright (c) 2013,2019 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$ 
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 
 *
 * $Revision: 359265 $
 */

import com.apama.jmon.Event;


public class Match extends Event {
	private static final long serialVersionUID = 17L;
    /** The id of the user in the application */
    public long userID ;

    /** The name or symbol of the stock */
    public  String stockName;

    /** The traded price of the stock which fell below the specified value */
    public double currentPrice;


    /**
     * No argument constructor
     */
    public Match() {
	this(0, "", 0);
    }


    /**
     * Construct a Match object and set the instance variables
     * 
     * @param  userID          The id of the user in the application 
     * @param  stockName       The name or symbol of the stock      
     * @param  currentPrice    The traded price of the stock which fell 
     *                         below the specified value 
     */
    public Match(long userID, String stockName, double currentPrice){
	this.userID = userID;
	this.stockName = stockName;
	this.currentPrice = currentPrice;
    }

}

