# Sample PySys testcase
# Copyright (c) 2015-2016, 2018 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors. 
# Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG 

from pysys.constants import *
from apama.basetest import ApamaBaseTest
from apama.correlator import CorrelatorHelper

class PySysTest(ApamaBaseTest):

	def execute(self):
		# create the first correlator helper, start it and inject the log monitor
		correlator1 = CorrelatorHelper(self, name='correlator1')
		correlator1.start(logfile='correlator1.log')
		correlator1.injectEPL(filenames='log.mon')
	
		# create the second correlator helper, start it and inject the forward monitor
		correlator2 = CorrelatorHelper(self, name='correlator2')
		correlator2.start(logfile='correlator2.log')
		correlator2.injectEPL(filenames='forward.mon')
	
		# connect correlator2 as a source of events to correlator1
		correlator1.connect(source=correlator2, channel='TestChannel')
		
		# send some events to correlator2 to be forwarded to correlator1
		correlator2.send(filenames='forward.evt')
		
		# wait for signals in correlator1 logfile
		self.waitForGrep(file='correlator1.log', expr='Received ForwardEvent', condition='>=5')
		
		# disconnect correlator2 as a source of events
		correlator1.disconnect(source=correlator2, channel='TestChannel', arguments=['-v'])
	
		# send some more events to correlator2
		correlator2.send(filenames='forward.evt')
		

	def validate(self):
		# check the first batch of events were received by correlator1
		exprList=[]
		exprList.append('LogMonitor \[\d+\] Received ForwardEvent, ForwardEvent\("Source correlator","Message 1"\)')
		exprList.append('LogMonitor \[\d+\] Received ForwardEvent, ForwardEvent\("Source correlator","Message 2"\)')
		exprList.append('LogMonitor \[\d+\] Received ForwardEvent, ForwardEvent\("Source correlator","Message 3"\)')
		exprList.append('LogMonitor \[\d+\] Received ForwardEvent, ForwardEvent\("Source correlator","Message 4"\)')
		exprList.append('LogMonitor \[\d+\] Received ForwardEvent, ForwardEvent\("Source correlator","Message 5"\)')
		self.assertOrderedGrep(file='correlator1.log', exprList=exprList)
		
		# check the second batch of events were not received by correlator1
		self.assertLineCount(file='correlator1.log', expr='Received ForwardEvent', condition='==5')

